package com.qmth.distributed.print.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.qmth.distributed.print.business.entity.GradeBatch;
import com.qmth.distributed.print.business.entity.GradeBatchPaper;
import com.qmth.distributed.print.business.entity.TBSyncTask;
import com.qmth.distributed.print.business.enums.GradeAnalyzePaperStatusEnum;
import com.qmth.distributed.print.business.service.DataSyncReportService;
import com.qmth.distributed.print.business.service.GradeBatchPaperService;
import com.qmth.distributed.print.business.service.GradeBatchService;
import com.qmth.distributed.print.business.service.TBSyncTaskService;
import com.qmth.teachcloud.common.entity.BasicSemester;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.enums.TaskResultEnum;
import com.qmth.teachcloud.common.enums.TaskStatusEnum;
import com.qmth.teachcloud.common.service.BasicSemesterService;
import com.qmth.teachcloud.common.sync.TeachCloudReportTaskUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;

/**
 * 同步服务类
 */
@Service
public class DataSyncReportServiceImpl implements DataSyncReportService {

    private static final Logger log = LoggerFactory.getLogger(DataSyncReportServiceImpl.class);

    @Resource
    private BasicSemesterService basicSemesterService;

    @Resource
    private TBSyncTaskService tbSyncTaskService;

    @Resource
    private GradeBatchService gradeBatchService;

    @Resource
    private GradeBatchPaperService gradeBatchPaperService;

    @Resource
    private TeachCloudReportTaskUtils teachCloudReportTaskUtils;

    @Async
    @Override
    public void syncGradeBatch(GradeBatch gradeBatch, TBSyncTask tbSyncTask) {
        // 同步初始参数
        TaskResultEnum result = null;
        TaskStatusEnum status;
        String errorMessage = null;
        Long schoolId = tbSyncTask.getSchoolId();
        Long semesterId = gradeBatch.getSemesterId();
        try {
            // 同步中
            status = TaskStatusEnum.RUNNING;
            tbSyncTaskService.updateStatusAndResultById(tbSyncTask.getId(), null, status, null, null);

            // 同步学期
            BasicSemester basicSemester = basicSemesterService.getById(semesterId);
            if (basicSemester == null) {
                throw ExceptionResultEnum.ERROR.exception("学期不存在");
            }
            Long syncSemesterId = teachCloudReportTaskUtils.syncSemester(schoolId, basicSemester.getName(), basicSemester.getStartTime(), basicSemester.getEndTime(), true);
            if (syncSemesterId <= 0L) {
                throw ExceptionResultEnum.ERROR.exception("学期同步失败");
            }

            // 同步考试（分析批次）
            Long examId = teachCloudReportTaskUtils.syncExam(schoolId, gradeBatch.getThirdExamId(), gradeBatch.getBatchName(), gradeBatch.getBatchTime(), String.valueOf(syncSemesterId), true);
            if (examId <= 0L) {
                throw ExceptionResultEnum.ERROR.exception("考试同步失败");
            }

            // 同步分析课程
            QueryWrapper<GradeBatchPaper> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(GradeBatchPaper::getBatchId, gradeBatch.getId());
            List<GradeBatchPaper> gradeBatchPaperList = gradeBatchPaperService.list(queryWrapper);
            for (GradeBatchPaper gradeBatchPaper : gradeBatchPaperList) {
                String courseCode = gradeBatchPaper.getPaperNumber() + gradeBatchPaper.getPaperType();
                teachCloudReportTaskUtils.syncCourse(schoolId, String.valueOf(examId), courseCode, gradeBatchPaper.getPaperName(), gradeBatchPaper.getPaperType(), gradeBatchPaper.getTeachCollegeName(), gradeBatchPaper.getEnable());
            }

            // 更新批次表考试ID、状态
            UpdateWrapper<GradeBatch> updateWrapper = new UpdateWrapper<>();
            updateWrapper.lambda().set(GradeBatch::getThirdExamId, examId)
                    .set(GradeBatch::getStatus, GradeAnalyzePaperStatusEnum.SETTING_GRADE_PAPER_PARAM)
                    .eq(GradeBatch::getId, gradeBatch.getId());
            gradeBatchService.update(updateWrapper);
            // 任务结果
            result = TaskResultEnum.SUCCESS;
        } catch (Exception e) {
            result = TaskResultEnum.ERROR;
            errorMessage = e.getMessage();
        } finally {
            // 同步结束
            status = TaskStatusEnum.FINISH;
            tbSyncTaskService.updateStatusAndResultById(tbSyncTask.getId(), null, status, result, errorMessage);
        }
    }
}
